@@ -7,10 +7,9 @@ const PopupMenu = imports.ui.popupMenu;
 const Main = imports.ui.main;
 const Mainloop = imports.mainloop;
 const SignalManager = imports.misc.signalManager;
+const {findIndex} = imports.misc.util;
 
-const ICON_SCALE_FACTOR = 0.8; // for custom panel heights, 20 (default icon size) / 25 (default panel height)
-
-const DEFAULT_ICON_SIZE = 20;
+const NO_RESIZE_ROLES = ['shutter', 'filezilla'];
 
 // Override the factory and create an AppletPopupMenu instead of a PopupMenu
 class IndicatorMenuFactory extends PopupMenu.PopupMenuFactory {
@@ -50,6 +49,7 @@ class CinnamonSystrayApplet extends Applet.Applet {
         let manager;
 
         this.orientation = orientation;
+        this.icon_size = this.getPanelIconSize(St.IconType.FULLCOLOR) * global.ui_scale;
 
         if (this.orientation == St.Side.TOP || this.orientation == St.Side.BOTTOM) {
             manager = new Clutter.BoxLayout( { spacing: 2,
@@ -63,7 +63,6 @@ class CinnamonSystrayApplet extends Applet.Applet {
         this.actor.add_actor (this.manager_container);
         this.manager_container.show();
 
-        this._statusItems = [];
         this._shellIndicators = [];
         this.menuFactory = new IndicatorMenuFactory();
         this.menuManager = new PopupMenu.PopupMenuManager(this);
@@ -114,10 +113,7 @@ class CinnamonSystrayApplet extends Applet.Applet {
 
     _onIndicatorAdded(manager, appIndicator) {
         if (!(appIndicator.id in this._shellIndicators)) {
-            let size = null;
-            size = this._getIconSize(this._panelHeight / global.ui_scale);
-
-            let indicatorActor = appIndicator.getActor(size);
+            let indicatorActor = appIndicator.getActor(this.icon_size);
 
             this._shellIndicators.push({
                 id: appIndicator.id,
@@ -159,28 +155,6 @@ class CinnamonSystrayApplet extends Applet.Applet {
         }
     }
 
-    _getIconSize(ht) {
-        let size;
-        let disp_size = ht * ICON_SCALE_FACTOR;  // hidpi with largest panel, gets up to 80
-
-        if (disp_size < 22) {
-            size = 16;
-        } else if (disp_size < 32) {
-            size = 22;
-        } else if (disp_size < 48) {
-            size = 32;
-        } else if (disp_size < 64) {
-            size = 48;
-        } else if (disp_size < 96) {
-            size = 64;
-        } else if (disp_size < 128) {
-            size = 96;
-        } else {
-            size = 48;
-        }
-        return size;
-    }
-
     _onIndicatorRemoved(manager, appIndicator) {
         for (let i = 0; i < this._shellIndicators.length; i++) {
             if (this._shellIndicators[i].id === appIndicator.id) {
@@ -202,30 +176,40 @@ class CinnamonSystrayApplet extends Applet.Applet {
         }
     }
 
-    on_applet_removed_from_panel () {
+    on_applet_reloaded() {
+        global.trayReloading = true;
+    }
+
+    on_applet_removed_from_panel() {
         this._signalManager.disconnectAllSignals();
         this._removeIndicatorSupport();
     }
 
     on_applet_added_to_panel() {
-        Main.statusIconDispatcher.start(this.actor.get_parent().get_parent());
+        if (!global.trayReloading) {
+            Main.statusIconDispatcher.start(this.actor.get_parent().get_parent());
+        }
 
         this._signalManager.connect(Main.statusIconDispatcher, 'status-icon-added', this._onTrayIconAdded, this);
         this._signalManager.connect(Main.statusIconDispatcher, 'status-icon-removed', this._onTrayIconRemoved, this);
         this._signalManager.connect(Main.statusIconDispatcher, 'before-redisplay', this._onBeforeRedisplay, this);
         this._signalManager.connect(Main.systrayManager, "changed", Main.statusIconDispatcher.redisplay, Main.statusIconDispatcher);
         this._addIndicatorSupport();
+
+        if (global.trayReloading) {
+            global.trayReloading = false;
+            Main.statusIconDispatcher.redisplay();
+        }
     }
 
-    on_panel_height_changed() {
+    on_panel_icon_size_changed(size) {
+        this.icon_size = size * global.ui_scale;
         Main.statusIconDispatcher.redisplay();
-        let size = null;
-        size = this._getIconSize(this._panelHeight / global.ui_scale);
 
         for (let i = 0; i < this._shellIndicators.length; i++) {
             let indicator = Main.indicatorManager.getIndicatorById(this._shellIndicators[i].id);
             if (indicator) {
-                this._shellIndicators[i].instance.setSize(size);
+                this._shellIndicators[i].instance.setSize(this.icon_size);
             }
         }
     }
@@ -234,10 +218,6 @@ class CinnamonSystrayApplet extends Applet.Applet {
         // Mark all icons as obsolete
         // There might still be pending delayed operations to insert/resize of them
         // And that would crash Cinnamon
-        for (var i = 0; i < this._statusItems.length; i++) {
-            this._statusItems[i].obsolete = true;
-        }
-        this._statusItems = [];
 
         let children = this.manager_container.get_children().filter(function(child) {
             // We are only interested in the status icons and apparently we can not ask for
@@ -261,105 +241,45 @@ class CinnamonSystrayApplet extends Applet.Applet {
 
             global.log("Adding systray: " + role + " (" + icon.get_width() + "x" + icon.get_height() + "px)");
 
-            if (icon.get_parent())
-                icon.get_parent().remove_child(icon);
+            let parent = icon.get_parent();
+            if (parent) parent.remove_child(icon);
 
-            icon.obsolete = false;
-            this._statusItems.push(icon);
-
-            if (["pidgin"].indexOf(role) != -1) {
-                // Delay pidgin insertion by 10 seconds
-                // Pidgin is very weird.. it starts with a small icon
-                // Then replaces that icon with a bigger one when the connection is established
-                // Pidgin can be fixed by inserting or resizing after a delay
-                // The delay is big because resizing/inserting too early
-                // makes pidgin invisible (in absence of disk cache).. even if we resize/insert again later
-                this._insertStatusItemLater(role, icon, -1, 10000);
-            } else if (["shutter", "filezilla", "dropbox", "thunderbird", "unknown", "blueberry-tray.py", "mintupdate.py"].indexOf(role) != -1) {
-                // Delay insertion by 1 second
-                // This fixes an invisible icon in the absence of disk cache for : shutter
-                // filezilla, dropbox, thunderbird, blueberry, mintupdate are known to show up in the wrong size or position, this should fix them as well
-                // Note: as of Oct 2015, the dropbox systray is calling itself "unknown"
-                this._insertStatusItemLater(role, icon, -1, 1000);
-            } else {
-                // Delay all other apps by 1 second...
-                // For many of them, we don't need to do that,
-                // It's a small delay though and that fixes most buggy apps
-                // And we're far from having an exhaustive list of them..
-                this._insertStatusItemLater(role, icon, -1, 1000);
-            }
+            this._insertStatusItem(role, icon);
 
         } catch (e) {
             global.logError(e);
         }
     }
 
-    _insertStatusItemLater(role, icon, position, delay) {
-        // Inserts an icon in the systray after a delay (useful for buggy icons)
-        // Delaying the insertion of pidgin by 10 seconds for instance is known to fix it on empty disk cache
-        let timerId = Mainloop.timeout_add(delay, Lang.bind(this, function() {
-            this._insertStatusItem(role, icon, position);
-            Mainloop.source_remove(timerId);
-        }));
-    }
-
     _onTrayIconRemoved(o, icon) {
-        icon.obsolete = true;
-        for (var i = 0; i < this._statusItems.length; i++) {
-            if (this._statusItems[i] == icon) {
-                this._statusItems.splice(i, 1);
-            }
-        }
-
-        if (icon.get_parent() == this.manager_container) {
+        if (icon.get_parent() === this.manager_container) {
             this.manager_container.remove_child(icon);
         }
 
         icon.destroy();
     }
 
-    _insertStatusItem(role, icon, position) {
-        if (icon.obsolete == true) {
+    _insertStatusItem(role, icon) {
+        if (icon.is_finalized()) {
             return;
         }
-        let children = this.manager_container.get_children().filter(function(child) {
-            // We are only interested in the status icons and apparently we can not ask for
-            // child instanceof CinnamonTrayIcon.
-            return (child.toString().indexOf("CinnamonTrayIcon") != -1);
-        });
-        let i;
-        for (i = children.length - 1; i >= 0; i--) {
-            let rolePosition = children[i]._rolePosition;
-            if (position > rolePosition) {
-                this.manager_container.insert_child_at_index(icon, i + 1);
-                break;
-            }
-        }
-        if (i == -1) {
-            // If we didn't find a position, we must be first
-            this.manager_container.insert_child_at_index(icon, 0);
-        }
-        icon._rolePosition = position;
+        this.manager_container.insert_child_at_index(icon, 0);
 
-        if (this._scaleMode) {
+        if (["skypeforlinux"].indexOf(role) != -1) {
+            let size = 16 * global.ui_scale;
+            icon.set_size(size, size);
+            global.log("Resize " + role + " with hardcoded size (" + icon.get_width() + "x" + icon.get_height() + "px)");
+        }
+        else {
             this._resizeStatusItem(role, icon);
-        } else {
-            icon.set_pivot_point(0.5, 0.5);
-            icon.set_scale((DEFAULT_ICON_SIZE * global.ui_scale) / icon.width,
-                           (DEFAULT_ICON_SIZE * global.ui_scale) / icon.height);
         }
     }
 
     _resizeStatusItem(role, icon) {
-        if (icon.obsolete == true) {
-            return;
-        }
-
-        if (["shutter", "filezilla"].indexOf(role) != -1) {
+        if (NO_RESIZE_ROLES.indexOf(role) > -1) {
             global.log("Not resizing " + role + " as it's known to be buggy (" + icon.get_width() + "x" + icon.get_height() + "px)");
         } else {
-            let size = this._getIconSize(this._panelHeight);
-            icon.set_size(size, size);
+            icon.set_size(this.icon_size, this.icon_size);
             global.log("Resized " + role + " with normalized size (" + icon.get_width() + "x" + icon.get_height() + "px)");
             //Note: dropbox doesn't scale, even though we resize it...
         }
